use std::net::TcpStream;
use std::io;
use ssh2::Session;
use rpassword::prompt_password;
use serde::{Deserialize, Serialize};
use serde_yaml::from_str;
use std::fs::{self, File};
use std::io::prelude::*;

#[derive(Debug, Deserialize, Serialize)]
struct Group {
    hosts: Vec<String>,
    commands: Vec<String>,
}

#[derive(Debug, Deserialize, Serialize)]
struct Inventory {
    groups: Vec<Group>,
}

fn main() {
    let username = read_username();
    let password = read_password();

    let inventory_file = read_inventory_file();
    let inventory = from_str::<Inventory>(&inventory_file).unwrap();

    // Create the output directory if it doesn't exist
    let output_dir = "output";
    fs::create_dir_all(output_dir).unwrap();

    for group in inventory.groups {
        println!("Executing commands on group {:?}", group.hosts);
        for host in group.hosts {
            let tcp = TcpStream::connect(format!("{}:22", host)).unwrap();
            let mut sess = Session::new().unwrap();
            sess.set_tcp_stream(tcp);
            sess.handshake().unwrap();

            match sess.userauth_password(&username, &password) {
                Ok(_) => {
                    for command in &group.commands {
                        let output = execute_command(&mut sess, command);
                        let output_filename = format!("{}/{}.txt", output_dir, host);
                        let mut file = File::create(&output_filename).unwrap();
                        file.write_all(output.as_bytes()).unwrap();

                        println!("Output saved to {}", output_filename);
                    }
                },
                Err(err) => {
                    eprintln!("Failed to authenticate to {}: {}", host, err);
                }
            }
        }
    }
}

fn read_username() -> String {
    print!("Username: ");
    let _ = io::stdout().flush();
    let mut username = String::new();
    let stdin = io::stdin();
    stdin.read_line(&mut username).unwrap();
    username.trim().to_string()
}

fn read_password() -> String {
    prompt_password("Enter password: ").unwrap()
}

fn read_inventory_file() -> String {
    print!("Enter inventory file path: ");
    let _ = io::stdout().flush();
    let mut path = String::new();
    let stdin = io::stdin();
    stdin.read_line(&mut path).unwrap();
    let path = path.trim();

    let mut file = std::fs::File::open(path).unwrap();
    let mut inventory = String::new();
    file.read_to_string(&mut inventory).unwrap();
    inventory
}

fn execute_command(sess: &mut Session, command: &str) -> String {
    let mut channel = sess.channel_session().unwrap();
    channel.exec(command).unwrap();

    let mut output = String::new();
    channel.read_to_string(&mut output).unwrap();
    channel.wait_close().unwrap();
    output
}
